@@ -1,14 +1,13 @@
-## ----setup, include=FALSE-----------------------------------------------------
+## ----setup, include=FALSE-------------------------------------------
 set.seed(123)
 library(INLA)
 inla.setOption(num.threads="1:1")
 inla.setOption(smtp="taucs")
-inla.setOption(inla.mode="experimental")
 if (file.exists("myinit.R")) source("myinit.R")
 knitr::opts_chunk$set(echo = TRUE)
 knitr::opts_chunk$set(fig.path="figures/multinomial/")
 
-## ---- message = FALSE, warning = FALSE----------------------------------------
+## ---- message = FALSE, warning = FALSE------------------------------
 # number of observations 
 n = 10
 
@@ -22,13 +21,13 @@ Ntrials = sample(100:1000, n, replace=TRUE)
 # Observations
 y = rbinom(n, size=Ntrials, prob = prob)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 lprec = 25
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 sqrt(1/exp(-lprec))
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 mult.model = inla(y ~ -1 + intercept + x,
          data = data.frame(y, x, Ntrials, intercept = rep(1, n)),
          family = "binomial", Ntrials = Ntrials,
@@ -36,7 +35,7 @@ mult.model = inla(y ~ -1 + intercept + x,
          control.inla = list(tolerance = 1E-12))
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # initialize
 Y = intercept = xx = c()
 for(i in 1:n) {
@@ -48,10 +47,10 @@ for(i in 1:n) {
 }
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 phi = rep(1:n, each=2)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 pois.model = inla(Y ~ -1
           + intercept
           + f(phi, model="iid",
@@ -63,22 +62,22 @@ pois.model = inla(Y ~ -1
           control.inla = list(tolerance = 1E-12))
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 corr.factor = - sum(dnorm(pois.model$summary.random$phi$mode,
                           sd = sqrt(1/exp(-lprec)), log=TRUE)) + sum(log(Ntrials))
 print(mult.model$mlik)
 print(pois.model$mlik + corr.factor)
 
-## ---- message=F---------------------------------------------------------------
+## ---- message=F-----------------------------------------------------
 library(Ecdat)
 library(deldir)
 library(sp)
-library(rgeos)
+library(sf)
 library(mvtnorm)
 library(gridExtra)
 library(mlogit)
 
-## ---- message=FALSE-----------------------------------------------------------
+## ---- message=FALSE-------------------------------------------------
 beta = -0.3
 deltas = c(1, 4, 3)
 gammas = c(0.3, 0.2, 0.4)
@@ -100,7 +99,7 @@ W.C = abs(rnorm(n, mean = 1))
 Z = rnorm(n, 20, 3)
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 Multinom.sample = function(N){
   Y = matrix(NA, ncol = 3, nrow = n)
   for(i in 1:n){
@@ -118,11 +117,11 @@ Multinom.sample = function(N){
   return(Y)
 }
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 head(Multinom.sample(1), 5)
 head(Multinom.sample(100), 5)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 Y = Multinom.sample(1)
 df = data.frame(cbind(Y, X.A, X.B, X.C, W.A, W.B, W.C, Z))
 
@@ -149,22 +148,22 @@ Data.structure = function(df){
 
 round(head(Data.structure(df)),3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 formula = Y ~ -1 + X + W.A + W.B + W.C + 
   f(phi, initial = -10, fixed = T) +
   f(alt.idx, Z, fixed = T, constr = T)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 Data = Data.structure(df)
 model = inla(formula, data = Data, family = 'Poisson')
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 result = rbind(model$summary.fixed[1:5], model$summary.random$alt.idx[2:6])
 result = cbind(result, true = param)
 row.names(result) = c("beta","delta.A","delta.B","delta.C","gamma.A","gamma.B","gamma.C" )
 round(result,3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 diff.result = 
   cbind("0.025quant"= diff(model$summary.random$alt.idx$`0.025quant`),
       "0.5quant" = diff(model$summary.random$alt.idx$`0.5quant`),
@@ -173,7 +172,7 @@ diff.result =
 row.names(diff.result) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
 round(diff.result,3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 random.effect = rnorm(n)
 Multinom.sample.rand = function(N, random.effect){
   Y = matrix(NA, ncol = 3, nrow = n)
@@ -197,26 +196,26 @@ Y.rand1 = Multinom.sample.rand(100, random.effect)
 df.rand1 = data.frame(cbind(Y.rand1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
 Data.rand1 = Data.structure(df.rand1)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 rand.idx = rep(NA, n*3)
 rand.idx[seq(1,n*3, by = 3)] = seq(1,n)
 Data.rand1$rand.idx = rand.idx
 round(head(Data.rand1),3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 formula.rand1 = Y ~ -1 + X + W.A + W.B + W.C + 
   f(phi, initial = -10, fixed = T) +
   f(alt.idx, Z, fixed = T, constr = T) + 
   f(rand.idx, model = "iid")  #random effect
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 model.rand1 = inla(formula.rand1, data = Data.rand1, family = 'Poisson')
 result.rand1 = rbind(model.rand1$summary.fixed[1:5])
 result.rand1 = cbind(result.rand1, true = param[1:4])
 row.names(result.rand1) = c("beta","delta.A","delta.B","delta.C")
 round(result.rand1,3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 diff.result.rand1 = 
   cbind("0.025quant"= diff(model.rand1$summary.random$alt.idx$`0.025quant`),
       "0.5quant" = diff(model.rand1$summary.random$alt.idx$`0.5quant`),
@@ -225,10 +224,10 @@ diff.result.rand1 =
 row.names(diff.result.rand1) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
 round(diff.result.rand1,3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 mean((random.effect - model.rand1$summary.random$rand.idx$`0.5quant`)^2)
 
-## ---- fig.height=2.5----------------------------------------------------------
+## ---- fig.height=2.5------------------------------------------------
 #  Generate the random effect
 random.walk1 = rep(NA, n)
 random.walk1[1] = 0
@@ -242,7 +241,7 @@ df.rw1 = data.frame(cbind(Y.rw1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
 Data.rw1 = Data.structure(df.rw1)
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 rw1.idx = rep(NA, n*3)
 rw1.idx[seq(1,n*3, by = 3)] = seq(1,n)
 Data.rw1$rw1.idx = rw1.idx
@@ -259,12 +258,12 @@ plot(random.walk1 - mean(random.walk1), type = 'l',
      main = 'Random walk estimate', ylab = 'values', xlab = 'time')
 points(model.rw1$summary.random$rw1.idx$`0.5quant`, type = 'l', col = 'red')
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 x.loc = runif(n)
 y.loc = runif(n)
 loc = cbind("x.loc" = x.loc, "y.loc" = y.loc)
 
-## ---- message=FALSE-----------------------------------------------------------
+## ---- message=FALSE-------------------------------------------------
 
 voronoi.polygons <- function(x) {
   require(deldir)
@@ -288,7 +287,7 @@ voronoi.polygons <- function(x) {
                                                                        function(x) slot(x, 'ID'))))
 }
 
-## ---- message=F---------------------------------------------------------------
+## ---- message=F-----------------------------------------------------
 # number of regions
 n.reg = 50
 # corners
@@ -298,7 +297,7 @@ points = rbind(boundaries, cbind(runif(n.reg - 4), runif(n.reg - 4)))
 # generate Voronoi polygons
 vor = voronoi.polygons(points)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # Create an ID column for the regions, it will be our index
 vor@data$id = seq(1,n.reg)
 # Transform the locations in SpatialPoint
@@ -309,26 +308,31 @@ id.samples = over(pp,vor)$id
 check = as.integer(length(unique(id.samples)))
 check
 
-## ---- fig.cap=c("Partiotion of the region [0,1]x[0,1], each red point correspond to a different location"), fig.height=5----
+## ---- fig.cap=c("Partition of the region [0,1]x[0,1], each red point correspond to a different location"), fig.height=5----
 # plot the partition and the locations
 plot(vor)
 points(loc, pch = 21, bg = 2, cex = 0.7)
 
-## ---- message = F, warnings = F-----------------------------------------------
+## ---- message = F, warnings = F-------------------------------------
 # adjacency matrix
-ADJ = gTouches(vor, byid = T)
-# marginal variance of each random effect component
-S = diag(n.reg)
+ADJ <- as(st_touches(st_as_sf(vor)), "Matrix")
+# Unscaled precision matrix
+Q_unscaled <- Matrix::Diagonal(n.reg, colSums(ADJ)) - ADJ * 0.99 # CAR
+# marginal inverse standard deviation of each random effect component
+SD_inv <- Matrix::Diagonal(n.reg, 1.0^-1)
+# Std.dev. from the unscaled precision
+SD_scale <- Matrix::Diagonal(n.reg, diag(inla.qinv(Q_unscaled))^0.5)
 # covariance matrix
-cov.matrix = solve(diag(n.reg) - ADJ)%*%S
+prec.matrix <- (SD_inv %*% SD_scale) %*% Q_unscaled %*% (SD_scale %*% SD_inv)
+prec.matrix <- (prec.matrix + t(prec.matrix)) / 2
 # generate the random effect and add it to the SpatialPolygonDataFrame
-vor@data$rand.eff = as.numeric(rmvnorm(1,sigma = cov.matrix))
+vor@data$rand.eff <- as.numeric(inla.qsample(1, prec.matrix, seed = 1234))
 
 
-## ---- fig.cap=c("Random effect on the partition"), fig.height=4---------------
+## ---- fig.cap=c("Random effect on the partition"), fig.height=4-----
 spplot(vor, 'rand.eff', col.regions = terrain.colors(32))
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # generate random effect vector
 random.effect = vor@data$rand.eff[id.samples]
 # generate sample
@@ -337,7 +341,7 @@ Y.spatial = Multinom.sample.rand(N = 100, random.effect)
 df.spatial = data.frame(cbind(Y.spatial, X.A, X.B, X.C, W.A, W.B, W.C, Z))
 Data.spatial = Data.structure(df.spatial)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # create and add index column
 rand.eff.idx = rep(NA, n*3)
 rand.eff.idx[seq(1,n*3, by = 3)] = id.samples
@@ -383,7 +387,7 @@ grid.arrange(spplot(vor, 'rand.eff', at = seq(-2.5, 2.5, length.out = 32),
                     col.regions = terrain.colors(32), main = "Residuals"),
              nrow = 1)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 mesh = inla.mesh.2d(loc, max.edge = c(0.02, 0.2), cutoff = 0.02)
 
 ## ---- fig.cap = c("Triangulation of the study area [0,1]x[0,1], the red dots represent the data points locations"), fig.height=5----
@@ -392,7 +396,7 @@ par(mfrow = c(1,1))
 plot(mesh)
 points(loc, pch = 21, bg = 2)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # starting parametrization
 size <- min(c(diff(range(mesh$loc[, 1])), diff(range(mesh$loc[, 2]))))
 range <- size/5
@@ -406,15 +410,15 @@ tau0 <- 1/(sqrt(4 * pi) * kappa0 * sigma0)
 spde <- inla.spde2.matern(mesh, B.tau = cbind(log(tau0), -1, +1),
                           B.kappa = cbind(log(kappa0), 0, -1))
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 Q = inla.spde.precision(spde, theta = c(0,0))
 # the number of elements of Q is the square of the number of triangles 
 dim(Q)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 sample = inla.qsample(n = 2, Q, seed = 123)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # create A
 A = inla.spde.make.A(mesh = mesh,
                       loc = cbind(rep(loc[,1], each = 3), 
@@ -423,7 +427,7 @@ A = inla.spde.make.A(mesh = mesh,
 # select rows relative to the first alternatives
 A = A*rep(c(1,2,3) == 1, n)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 random.effect = matrix(drop(A%*%sample[,2]), ncol = 3, byrow = T)[,1]
 
 ## ---- fig.cap=c("The right figure represents the continuos spatial effect, the left figure represents the value of the random effect on the sample locations" ), eval.after='fig.cap'----
@@ -441,7 +445,7 @@ rbPal <- colorRampPalette(heat.colors(100))
 Col <- rbPal(100)[as.numeric(cut(random.effect, breaks = 100))]
 plot(loc, pch = 20, col = Col, xlab = '', ylab = '')
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # generate sample
 Y.spde = Multinom.sample.rand(N = 100, random.effect)
 # construct data set
@@ -449,7 +453,7 @@ df.spde = data.frame(cbind(Y.spde, X.A, X.B, X.C, W.A, W.B, W.C, Z))
 Data.spde = Data.structure(df.spde)
 round(head(Data.spde),3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # index for the random effect
 s_index <- inla.spde.make.index(name="spatial.field",
                                 n.spde= spde$n.spde)
@@ -460,7 +464,7 @@ stack = inla.stack(data = list(Y = Data.spde$Y),
                    A = list(A, 1))
 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 formula.spde = Y ~ -1 + X + W.A + W.B + W.C + 
   f(phi, initial = -10, fixed = T) +
   f(alt.idx, Z, fixed = T, constr = T) +
@@ -477,7 +481,7 @@ model.spde <- inla(formula.spde,
 
 round(model.spde$internal.summary.hyperpar$mode, 3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 # estimates on the mesh locations
 spde.mesh.est = model.spde$summary.random$spatial.field$`0.5quant`
 # estimates on the data locations
@@ -486,7 +490,7 @@ spde.loc.est = matrix(drop(A%*%spde.mesh.est),
 # Mean Square Error
 mean((spde.loc.est - random.effect)^2)
 
-## ---- fig.cap=c("Recostruction of the spatial random field")------------------
+## ---- fig.cap=c("Recostruction of the spatial random field")--------
 par(mfrow = c(1,2))
 
 # plot the estimates
@@ -500,11 +504,11 @@ image(proj$x, proj$y, sample_proj , xlim = c(0,1), ylim = c(0,1),
       xlab = '',ylab = '', main = "True")
 
 
-## ---- message=F---------------------------------------------------------------
+## ---- message=F-----------------------------------------------------
 data("Yogurt")
 head(Yogurt)
 
-## ---- echo = F----------------------------------------------------------------
+## ---- echo = F------------------------------------------------------
 n = nrow(Yogurt)
 Data = matrix(NA, ncol = 5, nrow = n*4)
 for(i in 1:n){
@@ -529,22 +533,22 @@ Data$price = Data$price/100
 Data$phi = as.factor(Data$phi)
 Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 head(Data)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 formula = Y ~ -1 + price + feat + alpha.idx +
   f(phi, initial = -10, fixed = T) 
   
 
 model = inla(formula, data = Data, family = 'Poisson') 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 results = model$summary.fixed[,1:5]
 rownames(results) = c("price", "feat", "dannon", "hiland", "weight", "yoplait")
 round(results,3)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 Data$price.class = inla.group(Data$price, n = 12, method = "cut")
 
 formula.price = Y ~ -1 + feat + alpha.idx +
@@ -557,13 +561,13 @@ res = model.price$summary.fixed[,1:5]
 rownames(res) = c('feat', 'dannon','hiland','weight','yoplait')
 round(model.price$summary.random$price.class[,1:5],3)
 
-## ---- fig.cap=c("Relation between the coefficient and the price level")-------
+## ---- fig.cap=c("Relation between the coefficient and the price level")----
 plot(model.price$summary.random$price.class$ID,
      model.price$summary.random$price.class$mean, type = 'l', 
      ylab = expression(beta[1]),
      xlab = 'price')
 
-## ---- echo = F----------------------------------------------------------------
+## ---- echo = F------------------------------------------------------
 Data = matrix(NA, ncol = 8, nrow = n*4)
 for(i in 1:n){
   choice = rep(0, 4)
@@ -590,7 +594,7 @@ Data$phi = as.factor(Data$phi)
 Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)
 head(Data)
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 formula.feat = Y ~ -1 + price + feat.yoplait + feat.dannon + 
   feat.hiland + feat.weight + alpha.idx + 
   f(phi, initial = -10, fixed = T)  
@@ -598,6 +602,6 @@ formula.feat = Y ~ -1 + price + feat.yoplait + feat.dannon +
 
 model.feat = inla(formula.feat, data = Data, family = 'Poisson') 
 
-## -----------------------------------------------------------------------------
+## -------------------------------------------------------------------
 model.feat$summary.fixed[2:5, 1:5]
 
